within ;
package AC 
  
  annotation (uses(Modelica(version="2.2.2")),
  version="0.2.0 beta", versionDate="2008-01-15",
  preferedView="info", Documentation(info="<HTML>
This is a test implementation of complex time phasors, according to the paper:<br>
<b>Quasi-Stationary Modeling and Simulation of Electrical Circuits using Complex Phasors</b><br>
Anton Haumer, Christian Kral, Johannes V. Gragger, Hansjrg Kapeller<br>
<a href=\"http://www.modelica.org/events/modelica2008\">6<sup>th</sup> International Modelica Conference 2008</a>, Bielefeld, Germany
<p>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting & Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern<br>Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </dd>
</dl>
</p>
<p>
Copyright &copy; 2008 Anton Haumer.
</p>
</HTML>", revisions="<HTML>
  <ul>
  <li> v0.2.0 beta 2008/01/15 Anton Haumer<br>
       demo release for Modelica 2008</li>
  </ul>
</HTML>"));
  import AC.Types.Complex;
  
  package Examples 
    
    model TestConversionBlock 
      extends Modelica.Icons.Example;
      Blocks.ToComplex toComplex(FromPolar=true) 
        annotation (extent=[-40,-10; -20,10]);
      Blocks.FromComplex fromComplex(FromPolar=true) 
        annotation (extent=[0,-10; 20,10]);
      Modelica.Blocks.Sources.Ramp len(duration=1, offset=1E-6) 
        annotation (extent=[-80,10; -60,30]);
      Modelica.Blocks.Sources.Ramp phi(height=4*Modelica.Constants.pi, duration=1) 
        annotation (extent=[-80,-30; -60,-10]);
      annotation (Diagram(Text(
            extent=[-80,-60; 80,-80],
            style(color=3, rgbcolor={0,0,255}),
            string="plot fromComplex.im versus fromComplex.re")));
    equation 
      connect(len.y, toComplex.len) annotation (points=[-59,20; -50,20; -50,-4;
            -40,-4], style(color=74, rgbcolor={0,0,127}));
      connect(phi.y, toComplex.phi) annotation (points=[-59,-20; -50,-20; -50,
            -8; -40,-8], style(color=74, rgbcolor={0,0,127}));
      connect(toComplex.yP, fromComplex.uP) annotation (points=[-19,-6; 0,-6],
          style(color=58, rgbcolor={0,127,0}));
    end TestConversionBlock;
    
    model SeriesResonance 
      extends Modelica.Icons.Example;
      Modelica.Blocks.Sources.Constant V annotation (extent=[-100,40; -80,60]);
      Modelica.Blocks.Sources.Constant phi(k=0) 
                                           annotation (extent=[-100,10; -80,30]);
      Modelica.Blocks.Sources.Ramp f(
        height=2,
        duration=1,
        offset=1e-6) annotation (extent=[-100,-20; -80,0]);
      Blocks.ToComplex toComplex(FromPolar=true) annotation (extent=[-60,20;
            -40,40]);
      SinglePhase.Sources.VariableVoltageSource voltageSource 
        annotation (extent=[0,10; -20,30],    rotation=-90);
      AC.SinglePhase.Basic.Ground ground 
                          annotation (extent=[-20,-20; 0,0]);
      AC.SinglePhase.Basic.Resistor resistor(
                              R=0.1) 
        annotation (extent=[30,30; 50,50]);
      AC.SinglePhase.Basic.Inductor inductor(L=1/(2*Modelica.Constants.pi)) 
        annotation (extent=[60,30; 80,50]);
      AC.SinglePhase.Basic.Capacitor capacitor(C=1/(2*Modelica.Constants.pi)) 
        annotation (extent=[80,10; 100,30], rotation=-90);
      AC.SinglePhase.Sensors.CurrentSensor currentSensor 
                                          annotation (extent=[0,50; 20,30]);
      annotation (Diagram(Text(
            extent=[-80,-60; 80,-80],
            style(color=3, rgbcolor={0,0,255}),
            string=
                "plot currentSensor.yP.len and currentSensor.yP.phi versus f.y")));
    equation 
      connect(ground.p, voltageSource.n) annotation (points=[-10,0; -10,10],
          style(
          color=58,
          rgbcolor={0,127,0},
          fillColor=7,
          rgbfillColor={255,255,255},
          fillPattern=1));
      connect(voltageSource.p, currentSensor.p) annotation (points=[-10,30; -10,
            40; 0,40],   style(
          color=58,
          rgbcolor={0,127,0},
          fillColor=7,
          rgbfillColor={255,255,255},
          fillPattern=1));
      connect(currentSensor.n, resistor.p) annotation (points=[20,40; 30,40],
          style(
          color=58,
          rgbcolor={0,127,0},
          fillColor=7,
          rgbfillColor={255,255,255},
          fillPattern=1));
      connect(resistor.n, inductor.p) annotation (points=[50,40; 60,40], style(
          color=58,
          rgbcolor={0,127,0},
          fillColor=7,
          rgbfillColor={255,255,255},
          fillPattern=1));
      connect(inductor.n, capacitor.p) annotation (points=[80,40; 90,40; 90,30],
          style(
          color=58,
          rgbcolor={0,127,0},
          fillColor=7,
          rgbfillColor={255,255,255},
          fillPattern=1));
      connect(capacitor.n, ground.p) annotation (points=[90,10; 90,0; -10,0],
          style(color=58, rgbcolor={0,127,0}));
      connect(toComplex.yP, voltageSource.V) annotation (points=[-39,24; -20,24],
                             style(color=58, rgbcolor={0,127,0}));
      connect(V.y, toComplex.len) annotation (points=[-79,50; -70,50; -70,26;
            -60,26],
          style(color=74, rgbcolor={0,0,127}));
      connect(toComplex.phi, phi.y) annotation (points=[-60,22; -70,22; -70,20;
            -79,20],
          style(color=74, rgbcolor={0,0,127}));
      connect(f.y, voltageSource.f) annotation (points=[-79,-10; -30,-10; -30,
            16; -20,16],
                     style(color=74, rgbcolor={0,0,127}));
    end SeriesResonance;
    
    model ParallelResonance 
      extends Modelica.Icons.Example;
      Modelica.Blocks.Sources.Constant I annotation (extent=[-100,10; -80,30]);
      Modelica.Blocks.Sources.Constant phi(k=0) 
                                           annotation (extent=[-100,-20; -80,0]);
      Modelica.Blocks.Sources.Ramp f(
        height=2,
        duration=1,
        offset=1e-6) annotation (extent=[-100,40; -80,60]);
      Blocks.ToComplex toComplex(FromPolar=true) annotation (extent=[-60,0; -40,
            20]);
      SinglePhase.Sources.VariableCurrentSource currentSource 
        annotation (extent=[0,30; -20,10],    rotation=-90);
      AC.SinglePhase.Basic.Ground ground 
                          annotation (extent=[-20,-20; 0,0]);
      AC.SinglePhase.Basic.Resistor resistor(
                              R=10) 
        annotation (extent=[10,10; 30,30],    rotation=-90);
      AC.SinglePhase.Basic.Inductor inductor(L=1/(2*Modelica.Constants.pi)) 
        annotation (extent=[30,10; 50,30],   rotation=-90);
      AC.SinglePhase.Basic.Capacitor capacitor(C=1/(2*Modelica.Constants.pi)) 
        annotation (extent=[50,10; 70,30],  rotation=-90);
      AC.SinglePhase.Sensors.VoltageSensor voltageSensor 
                                          annotation (extent=[70,30; 90,10],  rotation=90);
      annotation (Diagram(Text(
            extent=[-80,-60; 80,-80],
            style(color=3, rgbcolor={0,0,255}),
            string=
                "plot voltageSensor.yP.len and voltageSensor.yP.phi versus f.y")));
    equation 
      connect(ground.p, currentSource.p) annotation (points=[-10,0; -10,10],
                      style(
          color=58,
          rgbcolor={0,127,0},
          fillColor=7,
          rgbfillColor={255,255,255},
          fillPattern=1));
      connect(currentSource.n, resistor.p) annotation (points=[-10,30; -10,40;
            20,40; 20,30],   style(
          color=58,
          rgbcolor={0,127,0},
          fillColor=7,
          rgbfillColor={255,255,255},
          fillPattern=1));
      connect(currentSource.n, inductor.p) annotation (points=[-10,30; -10,40;
            40,40; 40,30],                 style(
          color=58,
          rgbcolor={0,127,0},
          fillColor=7,
          rgbfillColor={255,255,255},
          fillPattern=1));
      connect(currentSource.n, capacitor.p) annotation (points=[-10,30; -10,40;
            60,40; 60,30], style(
          color=58,
          rgbcolor={0,127,0},
          fillColor=7,
          rgbfillColor={255,255,255},
          fillPattern=1));
      connect(currentSource.n, voltageSensor.p) annotation (points=[-10,30; -10,
            40; 80,40; 80,30], style(
          color=58,
          rgbcolor={0,127,0},
          fillColor=7,
          rgbfillColor={255,255,255},
          fillPattern=1));
      connect(resistor.n, ground.p) annotation (points=[20,10; 20,0; -10,0],
                  style(
          color=58,
          rgbcolor={0,127,0},
          fillColor=7,
          rgbfillColor={255,255,255},
          fillPattern=1));
      connect(inductor.n, ground.p) annotation (points=[40,10; 40,0; -10,0],
                                        style(
          color=58,
          rgbcolor={0,127,0},
          fillColor=7,
          rgbfillColor={255,255,255},
          fillPattern=1));
      connect(capacitor.n, ground.p) annotation (points=[60,10; 60,0; -10,0],
                  style(
          color=58,
          rgbcolor={0,127,0},
          fillColor=7,
          rgbfillColor={255,255,255},
          fillPattern=1));
      connect(voltageSensor.n, ground.p) annotation (points=[80,10; 80,0; -10,0],
                      style(
          color=58,
          rgbcolor={0,127,0},
          fillColor=7,
          rgbfillColor={255,255,255},
          fillPattern=1));
      connect(I.y,toComplex. len) annotation (points=[-79,20; -70,20; -70,6;
            -60,6],
          style(color=74, rgbcolor={0,0,127}));
      connect(toComplex.phi,phi. y) annotation (points=[-60,2; -70,2; -70,-10;
            -79,-10],
          style(color=74, rgbcolor={0,0,127}));
      connect(toComplex.yP, currentSource.I) annotation (points=[-39,4; -32,4;
            -32,16; -20,16], style(color=58, rgbcolor={0,127,0}));
      connect(f.y, currentSource.f) annotation (points=[-79,50; -30,50; -30,24;
            -20,24],
                style(color=74, rgbcolor={0,0,127}));
    end ParallelResonance;
    
    model AIMC 
      extends Modelica.Icons.Example;
      constant Modelica.SIunits.Angle pi = Modelica.Constants.pi;
      parameter Modelica.SIunits.Frequency fNominal=50 "nominal frequency";
      parameter Modelica.SIunits.Resistance Rs=0.03 
        "warm stator resistance per phase";
      parameter Modelica.SIunits.Inductance Lssigma=3*(1 - sqrt(1 - 0.0667))/(2*pi*fNominal) 
        "stator stray inductance per phase";
      parameter Modelica.SIunits.Inductance Lm=3*sqrt(1 - 0.0667)/(2*pi*fNominal) 
        "main field inductance";
      parameter Modelica.SIunits.Inductance Lrsigma=3*(1 - sqrt(1 - 0.0667))/(2*pi*fNominal) 
        "rotor stray inductance (equivalent three phase winding)";
      parameter Modelica.SIunits.Resistance Rr=0.04 
        "warm rotor resistance (equivalent three phase winding)";
      SinglePhase.Sources.VoltageSource voltageSource(f=fNominal, V=
            AC.Types.PolarVoltage(100, 0)) 
        annotation (extent=[-90,-10; -70,10], rotation=270);
      SinglePhase.Basic.Ground ground annotation (extent=[-90,-50; -70,-30]);
      SinglePhase.Sensors.CurrentSensor currentSensor 
        annotation (extent=[-80,40; -60,20]);
      annotation (Diagram(Text(
            extent=[-80,-60; 80,-80],
            style(color=3, rgbcolor={0,0,255}),
            string=
                "plot currentSensor.yP.len and powerSensor.y.re versus slip.y")));
      SinglePhase.Basic.Resistor rs(R=Rs) annotation (extent=[-50,20; -30,40]);
      SinglePhase.Basic.Inductor lssigma(L=Lssigma) 
                                               annotation (extent=[-20,20; 0,40]);
      SinglePhase.Sensors.PowerSensor powerSensor annotation (extent=[10,20; 30,40]);
      SinglePhase.Basic.Inductor lm(L=Lm) 
        annotation (extent=[20,-10; 40,10], rotation=270);
      SinglePhase.Basic.Inductor lrsigma(L=Lrsigma) 
        annotation (extent=[40,20; 60,40]);
      SinglePhase.Basic.VariableConductor gr_s 
        annotation (extent=[50,-10; 70,10], rotation=270);
      Modelica.Blocks.Math.Gain gain(k=1/Rr) annotation (extent=[50,50; 70,70]);
      Modelica.Blocks.Sources.Ramp slip(
        height=-1,
        duration=1,
        offset=1) annotation (extent=[20,50; 40,70]);
    equation 
      connect(voltageSource.n, ground.p) annotation (points=[-80,-10; -80,-30],
          style(
          color=58,
          rgbcolor={0,127,0},
          smooth=0));
      connect(voltageSource.p, currentSensor.p) annotation (points=[-80,10; -80,30],
          style(
          color=58,
          rgbcolor={0,127,0},
          smooth=0));
      connect(currentSensor.n, rs.p) annotation (points=[-60,30; -50,30], style(
          color=58,
          rgbcolor={0,127,0},
          smooth=0));
      connect(rs.n, lssigma.p) annotation (points=[-30,30; -20,30], style(
          color=58,
          rgbcolor={0,127,0},
          smooth=0));
      connect(lssigma.n, powerSensor.currentP) annotation (points=[0,30; 10,30],
          style(
          color=58,
          rgbcolor={0,127,0},
          smooth=0));
      connect(powerSensor.currentP, powerSensor.voltageP) annotation (points=[10,30;
            10,40; 20,40], style(
          color=58,
          rgbcolor={0,127,0},
          smooth=0));
      connect(powerSensor.currentN, lm.p) annotation (points=[30,30; 30,20; 30,
            10; 30,10],
                    style(
          color=58,
          rgbcolor={0,127,0},
          smooth=0));
      connect(lm.n, ground.p) annotation (points=[30,-10; 30,-30; -80,-30], style(
          color=58,
          rgbcolor={0,127,0},
          smooth=0));
      connect(powerSensor.voltageN, lm.n) annotation (points=[20,20; 20,-30; 30,
            -30; 30,-10],
          style(
          color=58,
          rgbcolor={0,127,0},
          smooth=0));
      connect(lrsigma.n, gr_s.p) annotation (points=[60,30; 60,10], style(
          color=58,
          rgbcolor={0,127,0},
          smooth=0));
      connect(gr_s.n, ground.p) annotation (points=[60,-10; 60,-30; -80,-30], style(
          color=58,
          rgbcolor={0,127,0},
          smooth=0));
      connect(gain.y, gr_s.G) annotation (points=[71,60; 80,60; 80,
            -2.02067e-015; 71,-2.02067e-015],
                               style(
          color=74,
          rgbcolor={0,0,127},
          smooth=0));
      connect(slip.y, gain.u) annotation (points=[41,60; 48,60], style(
          color=74,
          rgbcolor={0,0,127},
          smooth=0));
      connect(powerSensor.currentN, lrsigma.p) annotation (points=[30,30; 40,30],
          style(
          color=58,
          rgbcolor={0,127,0},
          smooth=0));
    end AIMC;
    
    model SM_R 
      extends Modelica.Icons.Example;
      constant Modelica.SIunits.Angle pi = Modelica.Constants.pi;
      parameter Modelica.SIunits.Frequency fNominal=50 "nominal frequency";
      parameter Modelica.SIunits.Resistance Rs=0.03 
        "warm stator resistance per phase";
      parameter Modelica.SIunits.Inductance Ld=1.6/(2*pi*fNominal) 
        "inductance per phase";
      SinglePhase.Sources.VoltageSource inducedVoltage(f=fNominal, V=
            AC.Types.PolarVoltage(100, 0)) 
        annotation (extent=[-90,-10; -70,10], rotation=270);
      SinglePhase.Basic.Ground ground annotation (extent=[-90,-50; -70,-30]);
      SinglePhase.Sensors.CurrentSensor currentSensor 
        annotation (extent=[-20,40; 0,20]);
      annotation (Diagram(Text(
            extent=[-80,-60; 80,-80],
            style(color=3, rgbcolor={0,0,255}),
            string="plot voltageSensor.yP.len versus currentSensor.yP.len ")));
      SinglePhase.Basic.Resistor rs(R=Rs) annotation (extent=[-50,20; -30,40]);
      SinglePhase.Basic.Inductor ld(L=Ld)      annotation (extent=[-80,20; -60,
            40]);
      Modelica.Blocks.Sources.Ramp logLoad(
        duration=1,
        offset=-50,
        height=100) 
                  annotation (extent=[90,-10; 70,10]);
      SinglePhase.Sensors.VoltageSensor voltageSensor 
        annotation (extent=[-10,-10; 10,10], rotation=270);
      Modelica.Blocks.Math.Exp exp annotation (extent=[60,-10; 40,10]);
      SinglePhase.Basic.VariableResistor load 
        annotation (extent=[10,-10; 30,10], rotation=270);
    equation 
      connect(inducedVoltage.n, ground.p) 
                                         annotation (points=[-80,-10; -80,-30],
          style(
          color=58,
          rgbcolor={0,127,0},
          smooth=0));
      connect(inducedVoltage.p, ld.p) annotation (points=[-80,10; -80,30],
          style(
          color=58,
          rgbcolor={0,127,0},
          smooth=0));
      connect(rs.p, ld.n) annotation (points=[-50,30; -60,30], style(
          color=58,
          rgbcolor={0,127,0},
          smooth=0));
      connect(rs.n, currentSensor.p) annotation (points=[-30,30; -20,30], style(
          color=58,
          rgbcolor={0,127,0},
          smooth=0));
      connect(currentSensor.n, voltageSensor.p) annotation (points=[0,30; 0,10;
            1.83697e-015,10], style(
          color=58,
          rgbcolor={0,127,0},
          smooth=0));
      connect(voltageSensor.n, ground.p) annotation (points=[-1.83697e-015,-10;
            0,-10; 0,-30; -80,-30], style(
          color=58,
          rgbcolor={0,127,0},
          smooth=0));
      connect(currentSensor.n, load.p) annotation (points=[0,30; 20,30; 20,10],
          style(
          color=58,
          rgbcolor={0,127,0},
          smooth=0));
      connect(load.n, ground.p) annotation (points=[20,-10; 20,-30; -80,-30],
          style(
          color=58,
          rgbcolor={0,127,0},
          smooth=0));
      connect(exp.y, load.R) annotation (points=[39,0; 35,0; 35,-2.02067e-015;
            31,-2.02067e-015], style(
          color=74,
          rgbcolor={0,0,127},
          smooth=0));
      connect(logLoad.y, exp.u) annotation (points=[69,0; 62,0],
          style(
          color=74,
          rgbcolor={0,0,127},
          smooth=0));
    end SM_R;
    
    model SM_L 
      extends Modelica.Icons.Example;
      constant Modelica.SIunits.Angle pi = Modelica.Constants.pi;
      parameter Modelica.SIunits.Frequency fNominal=50 "nominal frequency";
      parameter Modelica.SIunits.Resistance Rs=0.03 
        "warm stator resistance per phase";
      parameter Modelica.SIunits.Inductance Ld=1.6/(2*pi*fNominal) 
        "inductance per phase";
      SinglePhase.Sources.VoltageSource inducedVoltage(f=fNominal, V=
            AC.Types.PolarVoltage(100, 0)) 
        annotation (extent=[-90,-10; -70,10], rotation=270);
      SinglePhase.Basic.Ground ground annotation (extent=[-90,-50; -70,-30]);
      SinglePhase.Sensors.CurrentSensor currentSensor 
        annotation (extent=[-20,40; 0,20]);
      annotation (Diagram(Text(
            extent=[-80,-60; 80,-80],
            style(color=3, rgbcolor={0,0,255}),
            string="plot voltageSensor.yP.len versus currentSensor.yP.len ")));
      SinglePhase.Basic.Resistor rs(R=Rs) annotation (extent=[-50,20; -30,40]);
      SinglePhase.Basic.Inductor ld(L=Ld)      annotation (extent=[-80,20; -60,
            40]);
      Modelica.Blocks.Sources.Ramp logLoad(
        duration=1,
        offset=-50,
        height=100) 
                  annotation (extent=[90,-10; 70,10]);
      SinglePhase.Sensors.VoltageSensor voltageSensor 
        annotation (extent=[-10,-10; 10,10], rotation=270);
      Modelica.Blocks.Math.Exp exp annotation (extent=[60,-10; 40,10]);
      SinglePhase.Basic.VariableInductor load 
        annotation (extent=[10,-10; 30,10], rotation=270);
    equation 
      connect(inducedVoltage.n, ground.p) 
                                         annotation (points=[-80,-10; -80,-30],
          style(
          color=58,
          rgbcolor={0,127,0},
          smooth=0));
      connect(inducedVoltage.p, ld.p) annotation (points=[-80,10; -80,30],
          style(
          color=58,
          rgbcolor={0,127,0},
          smooth=0));
      connect(rs.p, ld.n) annotation (points=[-50,30; -60,30], style(
          color=58,
          rgbcolor={0,127,0},
          smooth=0));
      connect(rs.n, currentSensor.p) annotation (points=[-30,30; -20,30], style(
          color=58,
          rgbcolor={0,127,0},
          smooth=0));
      connect(currentSensor.n, voltageSensor.p) annotation (points=[0,30; 0,10;
            1.83697e-015,10], style(
          color=58,
          rgbcolor={0,127,0},
          smooth=0));
      connect(voltageSensor.n, ground.p) annotation (points=[-1.83697e-015,-10;
            0,-10; 0,-30; -80,-30], style(
          color=58,
          rgbcolor={0,127,0},
          smooth=0));
      connect(currentSensor.n, load.p) annotation (points=[0,30; 20,30; 20,10],
          style(
          color=58,
          rgbcolor={0,127,0},
          smooth=0));
      connect(load.n, ground.p) annotation (points=[20,-10; 20,-30; -80,-30],
          style(
          color=58,
          rgbcolor={0,127,0},
          smooth=0));
      connect(logLoad.y, exp.u) annotation (points=[69,0; 62,0],
          style(
          color=74,
          rgbcolor={0,0,127},
          smooth=0));
      connect(exp.y, load.L) annotation (points=[39,0; 34.9,0; 34.9,
            -1.98393e-015; 30.8,-1.98393e-015], style(
          color=74,
          rgbcolor={0,0,127},
          smooth=0));
    end SM_L;
  end Examples;
  
  package Blocks 
    
    block ToComplex 
      extends Modelica.Blocks.Interfaces.BlockIcon;
      parameter Boolean FromPolar = false 
        annotation(Evaluate=true);
      Modelica.Blocks.Interfaces.RealInput re if  not FromPolar 
        annotation (extent=[-120,60; -80,100]);
      Modelica.Blocks.Interfaces.RealInput im if not FromPolar 
        annotation (extent=[-120,20; -80,60]);
      Modelica.Blocks.Interfaces.RealInput len if FromPolar 
        annotation (extent=[-120,-60; -80,-20]);
      Modelica.Blocks.Interfaces.RealInput phi(
        redeclare type SignalType = Modelica.SIunits.Angle) if FromPolar 
        annotation (extent=[-120,-100; -80,-60]);
      Interfaces.ComplexOutput y annotation (extent=[100,50; 120,70]);
      Interfaces.PolarOutput yP annotation (extent=[100,-70; 120,-50]);
    protected 
      Modelica.Blocks.Interfaces.RealOutput re1 
                                               annotation (extent=[-62,78; -58,82]);
      Modelica.Blocks.Interfaces.RealOutput im1 
                                               annotation (extent=[-62,38; -58,42]);
      Modelica.Blocks.Interfaces.RealOutput len1 
                                                annotation (extent=[-62,-42; -58,-38]);
      Modelica.Blocks.Interfaces.RealOutput phi1 
                                                annotation (extent=[-62,-82; -58,-78]);
      annotation (Diagram, Icon);
    equation 
      if FromPolar then
        yP = Types.Polar(len1, phi1);
        y  = Complex.Constructors.fromPolar(yP);
      else
        y  = Complex(re1, im1);
        yP = Complex.toPolar(y);
      end if;
      connect(re, re1) annotation (points=[-100,80; -60,80], style(
          color=74,
          rgbcolor={0,0,127},
          fillColor=7,
          rgbfillColor={255,255,255},
          fillPattern=1));
      connect(im, im1) annotation (points=[-100,40; -60,40], style(
          color=74,
          rgbcolor={0,0,127},
          fillColor=7,
          rgbfillColor={255,255,255},
          fillPattern=1));
      connect(len, len1) annotation (points=[-100,-40; -60,-40], style(
          color=74,
          rgbcolor={0,0,127},
          fillColor=7,
          rgbfillColor={255,255,255},
          fillPattern=1));
      connect(phi, phi1) annotation (points=[-100,-80; -60,-80], style(
          color=74,
          rgbcolor={0,0,127},
          fillColor=7,
          rgbfillColor={255,255,255},
          fillPattern=1));
    end ToComplex;
    
    block FromComplex 
      extends Modelica.Blocks.Interfaces.BlockIcon;
      parameter Boolean FromPolar = false 
        annotation(Evaluate=true);
      annotation (Diagram, Icon);
      Modelica.Blocks.Interfaces.RealOutput re annotation (extent=[100,70; 120,90]);
      Modelica.Blocks.Interfaces.RealOutput im annotation (extent=[100,30; 120,50]);
      Modelica.Blocks.Interfaces.RealOutput len 
        annotation (extent=[100,-50; 120,-30]);
      Modelica.Blocks.Interfaces.RealOutput phi(
        redeclare type SignalType = Modelica.SIunits.Angle) annotation (extent=[100,-90; 120,-70]);
      Interfaces.ComplexInput u if not FromPolar 
        annotation (extent=[-120,40; -80,80]);
      Interfaces.PolarInput uP if FromPolar 
        annotation (extent=[-120,-80; -80,-40]);
    protected 
      Interfaces.ComplexOutput u1 
                                 annotation (extent=[-62,58; -58,62]);
      Interfaces.PolarOutput uP1 
                                annotation (extent=[-62,-62; -58,-58]);
    equation 
      if FromPolar then
        len = uP1.len;
        phi = uP1.phi;
        re  = Complex.'Real'(Complex.Constructors.fromPolar(uP1));
        im  = Complex.'Imag'(Complex.Constructors.fromPolar(uP1));
      else
        re  = u1.re;
        im  = u1.im;
        len = Complex.'abs'(u1);
        phi = Complex.arg(u1);
      end if;
      connect(uP, uP1) annotation (points=[-100,-60; -60,-60], style(
          color=58,
          rgbcolor={0,127,0},
          fillColor=7,
          rgbfillColor={255,255,255},
          fillPattern=1));
      connect(u, u1) annotation (points=[-100,60; -60,60], style(
          color=58,
          rgbcolor={0,127,0},
          fillColor=7,
          rgbfillColor={255,255,255},
          fillPattern=1));
    end FromComplex;
    
        block ToPolar "Output the sum of the elements of the input vector" 
          extends Modelica.Blocks.Interfaces.BlockIcon;
          parameter Integer nin=1 "Number of inputs";
          annotation (
            Icon(                           Text(extent=[-150, 150; 150, 110],
                  string="%name")),
            Diagram);
      Interfaces.ComplexInput u annotation (extent=[-140,-20; -100,20]);
      Interfaces.PolarOutput y annotation (extent=[100,-10; 120,10]);
        equation 
          y = Complex.toPolar(u);
        end ToPolar;
    
        block FromPolar "Output the sum of the elements of the input vector" 
          extends Modelica.Blocks.Interfaces.BlockIcon;
          parameter Integer nin=1 "Number of inputs";
          annotation (
            Icon(                           Text(extent=[-150, 150; 150, 110],
                  string="%name")),
            Diagram);
      Interfaces.PolarInput u annotation (extent=[-140,-20; -100,20]);
      Interfaces.ComplexOutput y annotation (extent=[100,-10; 120,10]);
        equation 
          y = Complex.Constructors.fromPolar(u);
        end FromPolar;
    
        block Sum "Output the sum of the elements of the input vector" 
          extends Modelica.Blocks.Interfaces.BlockIcon;
          parameter Integer nin=1 "Number of inputs";
          annotation (
            Icon(Line(points=[26, 42; -34, 42; 6, 2; -34, -38; 26, -38], style(
                    color=0, thickness=1)), Text(extent=[-150, 150; 150, 110],
                  string="%name")),
            Diagram);
      Interfaces.ComplexInput u[nin] annotation (extent=[-140,-20; -100,20]);
      Interfaces.ComplexOutput y annotation (extent=[100,-10; 120,10]);
        equation 
        //y = Complex.'sum'(u);
          y.re = sum(u.re);
          y.im = sum(u.im);
        end Sum;
    
    package Interfaces 
      
    connector ComplexSignal = AC.Types.Complex;
      
    connector ComplexInput = input ComplexSignal 
      annotation (defaultComponentName="u",
      Coordsys(extent=[-100, -100; 100, 100],
        grid=[1,1],
        component=[20,20],
          scale=0.2),
      Icon(coordinateSystem(extent=[-100,-100; 100,100]),
           Polygon(points=[-100,100; 100,0; -100,-100; -100,100], style(
              color=58,
              rgbcolor={0,127,0},
              fillColor=58,
              rgbfillColor={0,127,0}))),
      Diagram(Polygon(points=[0,50; 100,0; 0,-50; 0,50], style(
              color=58,
              rgbcolor={0,127,0},
              fillColor=58,
              rgbfillColor={0,127,0})),
                                      Text(
          extent=[-100,100; 99,60],
          string="%name",
            style(color=74, rgbcolor={0,0,127}))));
      
    connector ComplexOutput = output ComplexSignal 
      annotation (defaultComponentName="y",
      Coordsys(extent=[-100, -100; 100, 100],
        grid=[1,1],
        component=[20,20]),
      Icon(Polygon(points=[-100, 100; 100, 0; -100, -100; -100, 100], style(
            color=58,
            rgbcolor={0,127,0},
            fillColor=7,
            rgbfillColor={255,255,255}))),
      Diagram(Polygon(points=[-100,50; 0,0; -100,-50; -100,50], style(
            color=58,
            rgbcolor={0,127,0},
            fillColor=7,
            rgbfillColor={255,255,255})),
          Text(
          extent=[-100,100; 100,60],
          string="%name",
            style(color=74, rgbcolor={0,0,127}))));
      
    connector PolarSignal = AC.Types.Polar;
      
    connector PolarInput =   input PolarSignal 
      annotation (defaultComponentName="uP",
      Coordsys(extent=[-100, -100; 100, 100],
        grid=[1,1],
        component=[20,20],
          scale=0.2),
      Icon(coordinateSystem(extent=[-100,-100; 100,100]),
           Polygon(points=[-100,100; 100,0; -100,-100; -100,100], style(
              color=58,
              rgbcolor={0,127,0},
              fillColor=58,
              rgbfillColor={0,127,0})),
          Line(points=[99,1; -100,1], style(color=7, rgbcolor={255,255,255}))),
      Diagram(Polygon(points=[0,50; 100,0; 0,-50; 0,50], style(
              color=58,
              rgbcolor={0,127,0},
              fillColor=58,
              rgbfillColor={0,127,0})),
                                      Text(
          extent=[-100,100; 99,60],
          string="%name",
            style(color=74, rgbcolor={0,0,127})),
          Line(points=[100,0; 0,0], style(color=7, rgbcolor={255,255,255}))));
      
    connector PolarOutput =   output PolarSignal 
      annotation (defaultComponentName="yP",
      Coordsys(extent=[-100, -100; 100, 100],
        grid=[1,1],
        component=[20,20]),
      Icon(Polygon(points=[-100, 100; 100, 0; -100, -100; -100, 100], style(
            color=58,
            rgbcolor={0,127,0},
            fillColor=7,
            rgbfillColor={255,255,255})), Line(points=[99,0; -100,0], style(
                color=58, rgbcolor={0,127,0}))),
      Diagram(Polygon(points=[-100,50; 0,0; -100,-50; -100,50], style(
            color=58,
            rgbcolor={0,127,0},
            fillColor=7,
            rgbfillColor={255,255,255})),
          Text(
          extent=[-100,100; 100,60],
          string="%name",
            style(color=74, rgbcolor={0,0,127})),
          Line(points=[-1,0; -100,0; -100,1], style(color=58, rgbcolor={0,127,0}))));
    end Interfaces;
  end Blocks;
  
  package SinglePhase 
    
    package Basic 
      
      model Ground 
        parameter Boolean defineReference = false 
          annotation(Evaluate=true);
        parameter Modelica.SIunits.Frequency f = 0 
          annotation(Dialog(enable=defineReference));
        Modelica.SIunits.AngularVelocity w = der(p.ref.phi);
        AC.SinglePhase.Interfaces.PositivePin p 
          annotation (extent=[-10,90; 10,110]);
        annotation (Icon(
            Line(points=[-60,50; 60,50]),
            Line(points=[-40,30; 40,30]),
            Line(points=[-20,10; 20,10]),
            Line(points=[0,90; 0,50]),
            Text(extent=[100,-40; -100,0], string="%name")), Diagram);
      equation 
        Connections.potentialRoot(p.ref);
        if defineReference then
          w = 2*Modelica.Constants.pi*f;
        end if;
        p.v = Complex.'0'();
      end Ground;
      
      model Resistor 
        extends Interfaces.OnePort;
        parameter Modelica.SIunits.Resistance R=1;
        annotation (Icon(
            Text(extent=[100,-100; -100,-60], string="%name"),
            Line(points=[60,0; 90,0]),
            Line(points=[-90,0; -60,0]),
            Rectangle(extent=[-70,30; 70,-30],
              style(color=3, fillColor=7, fillPattern=1))), Diagram);
      equation 
        v = Complex.'*'(Complex(R, 0), i);
      end Resistor;
      
      model Conductor 
        extends Interfaces.OnePort;
        parameter Modelica.SIunits.Conductance G=1;
        annotation (Icon(
            Text(extent=[100,-100; -100,-60], string="%name"),
            Line(points=[60,0; 90,0]),
            Line(points=[-90,0; -60,0]),
            Rectangle(extent=[-70,30; 70,-30],
              style(color=3, fillColor=7, fillPattern=1))), Diagram);
      equation 
        i = Complex.'*'(Complex(G, 0), v);
      end Conductor;
      
      model Capacitor 
        extends Interfaces.OnePort;
        parameter Modelica.SIunits.Capacitance C=1;
        annotation (Icon(
            Text(extent=[100,-100; -100,-60], string="%name"),
            Line(points=[-14,28; -14,-28],   style(thickness=2)),
            Line(points=[14,28; 14,-28],   style(thickness=2)),
            Line(points=[-90,0; -14,0]),
            Line(points=[14,0; 90,0])), Diagram);
      equation 
        i = Complex.'*'(Complex(0, w*C), v);
      end Capacitor;
      
      model Inductor 
        extends Interfaces.OnePort;
        parameter Modelica.SIunits.Inductance L=1;
        annotation (Icon(
            Text(extent=[100,-100; -100,-60], string="%name"),
            Ellipse(extent=[-60,-15; -30,15]),
            Ellipse(extent=[-30,-15; 0,15]),
            Ellipse(extent=[0,-15; 30,15]),
            Ellipse(extent=[30,-15; 60,15]),
            Rectangle(extent=[-60,-30; 60,0], style(color=7, fillColor=7)),
            Line(points=[60,0; 90,0]),
            Line(points=[-90,0; -60,0])), Diagram);
      equation 
        v = Complex.'*'(Complex(0, w*L), i);
      end Inductor;
      
      model VariableResistor 
        extends Interfaces.OnePort;
        annotation (Icon(
            Text(extent=[100,-100; -100,-60], string="%name"),
            Line(points=[60,0; 90,0]),
            Line(points=[-90,0; -60,0]),
            Rectangle(extent=[-70,30; 70,-30],
              style(color=3, fillColor=7, fillPattern=1)),
            Line(points=[0,90; 0,30], style(
                color=3,
                rgbcolor={0,0,255},
                smooth=0))),                                Diagram);
        Modelica.Blocks.Interfaces.RealInput R(
          redeclare type SignalType = Modelica.SIunits.Resistance) 
          annotation (extent=[-20,90; 20,130],   rotation=-90);
      equation 
        v = Complex.'*'(Complex(R, 0), i);
      end VariableResistor;
      
      model VariableConductor 
        extends Interfaces.OnePort;
        annotation (Icon(
            Text(extent=[100,-100; -100,-60], string="%name"),
            Line(points=[60,0; 90,0]),
            Line(points=[-90,0; -60,0]),
            Rectangle(extent=[-70,30; 70,-30],
              style(color=3, fillColor=7, fillPattern=1)),
            Line(points=[0,90; 0,30], style(
                color=3,
                rgbcolor={0,0,255},
                smooth=0))),                                Diagram);
        Modelica.Blocks.Interfaces.RealInput G(
           redeclare type SignalType = Modelica.SIunits.Conductance) 
          annotation (extent=[-20,90; 20,130],   rotation=-90);
      equation 
        i = Complex.'*'(Complex(G, 0), v);
      end VariableConductor;
      
      model VariableCapacitor 
        extends Interfaces.OnePort;
        annotation (Icon(
            Text(extent=[100,-100; -100,-60], string="%name"),
            Line(points=[-14,28; -14,-28],   style(thickness=2)),
            Line(points=[14,28; 14,-28],   style(thickness=2)),
            Line(points=[-90,0; -14,0]),
            Line(points=[14,0; 90,0]),
            Line(points=[0,90; 0,30],   style(color=73))),
                                        Diagram);
        Modelica.Blocks.Interfaces.RealInput C(redeclare type SignalType = 
              Modelica.SIunits.Capacitance) 
          annotation (extent=[-20,90; 20,130],   rotation=-90);
      equation 
        i = Complex.'*'(Complex(0, w*C), v);
      end VariableCapacitor;
      
      model VariableInductor 
        extends Interfaces.OnePort;
        annotation (Icon(
            Text(extent=[100,-100; -100,-60], string="%name"),
            Ellipse(extent=[-60,-15; -30,15]),
            Ellipse(extent=[-30,-15; 0,15]),
            Ellipse(extent=[0,-15; 30,15]),
            Ellipse(extent=[30,-15; 60,15]),
            Rectangle(extent=[-60,-30; 60,0], style(color=7, fillColor=7)),
            Line(points=[60,0; 90,0]),
            Line(points=[-90,0; -60,0]),
            Line(points=[0,90; 0,8],   style(color=73))),
                                          Diagram);
        Modelica.Blocks.Interfaces.RealInput L(
           redeclare type SignalType = Modelica.SIunits.Inductance) 
          annotation (extent=[-20,88; 20,128],   rotation=-90);
      equation 
        v = Complex.'*'(Complex(0, w*L), i);
      end VariableInductor;
    end Basic;
    
    package Interfaces 
      
      connector Pin 
        Types.ComplexVoltage v "complex potential at the node";
        flow Types.ComplexCurrent i "complex current flowing into the pin";
        Types.Reference ref;
      end Pin;
      
      connector PositivePin 
        extends AC.SinglePhase.Interfaces.Pin;
        annotation (Diagram(              Text(
              extent=[-100,100; 100,60],
              style(
                color=3,
                rgbcolor={0,0,255},
                fillColor=3,
                rgbfillColor={0,0,255},
                fillPattern=1),
              string="%name"), Rectangle(extent=[-40,40; 40,-40], style(
                color=58,
                rgbcolor={0,127,0},
                fillColor=58,
                rgbfillColor={0,127,0}))),
                                  Icon(Rectangle(extent=[-100,100; 100,-100],
                style(
                color=58,
                rgbcolor={0,127,0},
                fillColor=58,
                rgbfillColor={0,127,0},
                fillPattern=1))));
      end PositivePin;
      
      connector NegativePin 
        extends AC.SinglePhase.Interfaces.Pin;
        annotation (Diagram(                  Text(
              extent=[-100,100; 100,60],
              style(
                color=3,
                rgbcolor={0,0,255},
                fillColor=3,
                rgbfillColor={0,0,255},
                fillPattern=1),
              string="%name"), Rectangle(extent=[-40,40; 40,-40], style(
                color=58,
                rgbcolor={0,127,0},
                fillColor=7,
                rgbfillColor={255,255,255},
                fillPattern=1))), Icon(Rectangle(extent=[-100,100; 100,-100],
                style(
                color=58,
                rgbcolor={0,127,0},
                fillColor=7,
                rgbfillColor={255,255,255},
                fillPattern=1))));
      end NegativePin;
      
      partial model TwoNode 
        Types.ComplexVoltage v = Complex.'-'(p.v, n.v);
        Types.ComplexCurrent i = p.i;
        Modelica.SIunits.AngularVelocity w = der(p.ref.phi);
        AC.SinglePhase.Interfaces.PositivePin p 
          annotation (extent=[-110,-10; -90,10]);
        AC.SinglePhase.Interfaces.NegativePin n 
          annotation (extent=[90,-10; 110,10]);
        annotation (Diagram);
      equation 
        Connections.branch(p.ref, n.ref);
        p.ref.phi = n.ref.phi;
      end TwoNode;
      
      partial model OnePort 
        extends TwoNode;
      equation 
        Complex.'+'(p.i, n.i) = Complex.'0'();
      end OnePort;
      
      partial model AbsoluteSensor 
        extends Modelica.Icons.RotationalSensor;
        Modelica.SIunits.AngularVelocity w = der(p.ref.phi);
        AC.SinglePhase.Interfaces.PositivePin p 
          annotation (extent=[-110, -10; -90, 10]);
        annotation (Diagram, Icon(
          Line(points=[-70,0; -94,0], style(color=0)),
            Text(
              extent=[-100,100; 100,70],
              string="%name",
              style(
                pattern=0,
                fillColor=79,
                rgbfillColor={170,85,255},
                fillPattern=1)),
            Line(points=[70,0; 80,0; 80,60; 100,60], style(
                color=58,
                rgbcolor={0,127,0},
                fillColor=7,
                rgbfillColor={255,255,255},
                fillPattern=1)),
            Line(points=[70,0; 80,0; 80,-60; 100,-60], style(
                color=58,
                rgbcolor={0,127,0},
                fillColor=7,
                rgbfillColor={255,255,255},
                fillPattern=1))));
        Blocks.Interfaces.ComplexOutput y annotation (extent=[100,50; 120,70]);
        Blocks.Interfaces.PolarOutput yP annotation (extent=[100,-70; 120,-50]);
      equation 
        p.i = Complex.'0'();
        yP = Complex.toPolar(y);
      end AbsoluteSensor;
      
      partial model RelativeSensor 
        extends Modelica.Icons.RotationalSensor;
        extends OnePort;
        annotation (Diagram, Icon(
            Line(points=[-70,0; -94,0],   style(color=0)),
            Line(points=[70,0; 94,0],   style(color=0)),
            Text(
              extent=[-100,100; 100,70],
              string="%name",
              style(
                pattern=0,
                fillColor=79,
                rgbfillColor={170,85,255},
                fillPattern=1)),
            Line(points=[0,-70; 0,-80; 60,-80; 60,-100], style(
                color=58,
                rgbcolor={0,127,0},
                fillColor=7,
                rgbfillColor={255,255,255},
                fillPattern=1)),
            Line(points=[0,-70; 0,-80; -60,-80; -60,-100], style(
                color=58,
                rgbcolor={0,127,0},
                fillColor=7,
                rgbfillColor={255,255,255},
                fillPattern=1))));
        Blocks.Interfaces.ComplexOutput y annotation (extent=[50,-120; 70,-100], rotation=-90);
        Blocks.Interfaces.PolarOutput yP annotation (extent=[-70,-120; -50,-100], rotation=-90);
      equation 
        yP = Complex.toPolar(y);
      end RelativeSensor;
      
      partial model Source 
        extends OnePort;
        annotation (Icon(
            Ellipse(extent=[-50,50; 50,-50], style(
                color=0,
                rgbcolor={0,0,0},
                fillColor=7,
                rgbfillColor={255,255,255})),
            Text(extent=[100,-100; -100,-60], string="%name"),
            Line(points=[-90,0; -50,0],style(color=0, rgbcolor={0,0,0})),
            Line(points=[50,0; 90,0],  style(color=0, rgbcolor={0,0,0}))), Diagram);
      equation 
        Connections.root(p.ref);
      end Source;
    end Interfaces;
    
    package Sensors 
      
      model PotentialSensor 
        extends AC.SinglePhase.Interfaces.AbsoluteSensor;
        annotation (Icon(
               Text(
              extent=[-29,-11; 30,-70],
              string="V",
              style(color=0))), Diagram);
      equation 
        y = p.v;
      end PotentialSensor;
      
      model VoltageSensor 
        extends AC.SinglePhase.Interfaces.RelativeSensor;
        annotation (Icon(
               Text(
              extent=[-29,-11; 30,-70],
              string="V",
              style(color=0))), Diagram);
      equation 
        i = Complex.'0'();
        y = v;
      end VoltageSensor;
      
      model CurrentSensor 
        extends AC.SinglePhase.Interfaces.RelativeSensor;
        annotation (Icon(
               Text(
              extent=[-29,-11; 30,-70],
              string="I",
              style(color=0))), Diagram);
      equation 
        v = Complex.'0'();
        y = i;
      end CurrentSensor;
      
      model PowerSensor 
        
        annotation (Diagram, Icon(
            Ellipse(extent=[-70,70; 70,-70],   style(color=0, fillColor=7)),
            Line(points=[0,100; 0,70], style(color=3, rgbcolor={0,0,255})),
            Line(points=[0,-70; 0,-100], style(color=3, rgbcolor={0,0,255})),
            Line(points=[-100,0; 100,0], style(color=3, rgbcolor={0,0,255})),
            Line(points=[0,70; 0,40],   style(color=0)),
            Line(points=[22.9,32.8; 40.2,57.3],   style(color=0)),
            Line(points=[-22.9,32.8; -40.2,57.3],   style(color=0)),
            Line(points=[37.6,13.7; 65.8,23.9],   style(color=0)),
            Line(points=[-37.6,13.7; -65.8,23.9],   style(color=0)),
            Line(points=[0,0; 9.02,28.6],   style(color=0)),
            Polygon(points=[-0.48,31.6; 18,26; 18,57.2; -0.48,31.6],     style(
                color=0,
                fillColor=0,
                fillPattern=1)),
            Ellipse(extent=[-5,5; 5,-5],   style(
                color=0,
                gradient=0,
                fillColor=0,
                fillPattern=1)),
               Text(
              extent=[-29,-11; 30,-70],
              style(color=0),
              string="P"),
            Line(points=[-60,-100; -60,-36], style(
                color=58,
                rgbcolor={0,127,0},
                fillColor=7,
                rgbfillColor={255,255,255},
                fillPattern=1)),
            Line(points=[60,-100; 60,-36], style(
                color=58,
                rgbcolor={0,127,0},
                fillColor=7,
                rgbfillColor={255,255,255},
                fillPattern=1))));
        AC.SinglePhase.Interfaces.PositivePin currentP 
          annotation (extent=[-110,-10; -90,10]);
        AC.SinglePhase.Interfaces.NegativePin currentN 
          annotation (extent=[90,-10; 110,10]);
        AC.SinglePhase.Interfaces.PositivePin voltageP 
          annotation (extent=[-10,90; 10,110]);
        AC.SinglePhase.Interfaces.NegativePin voltageN 
          annotation (extent=[-10,-110; 10,-90]);
        output Types.ComplexCurrent i = currentP.i;
        output Types.ComplexVoltage v = Complex.'-'(voltageP.v, voltageN.v);
        Blocks.Interfaces.ComplexOutput y 
          annotation (extent=[50,-120; 70,-100], rotation=-90);
        Blocks.Interfaces.PolarOutput yP 
          annotation (extent=[-70,-120; -50,-100], rotation=-90);
      equation 
        Connections.branch(currentP.ref, currentN.ref);
        currentP.ref.phi = currentN.ref.phi;
        Connections.branch(voltageP.ref, voltageN.ref);
        voltageP.ref.phi = voltageN.ref.phi;
        Complex.'+'(currentP.i, currentN.i) = Complex.'0'();
        Complex.'-'(currentP.v, currentN.v) = Complex.'0'();
        voltageP.i = Complex.'0'();
        voltageN.i = Complex.'0'();
      //P + j*Q = v * conj(i);
        y  = Complex.'*'(v, Complex.conj(i));
        yP = Complex.toPolar(y);
      end PowerSensor;
    end Sensors;
    
    package Sources 
      
      model VoltageSource 
        extends AC.SinglePhase.Interfaces.Source;
        parameter Modelica.SIunits.Frequency f = 1;
        parameter AC.Types.PolarVoltage V = AC.Types.PolarVoltage(1, 0);
        annotation (Icon(
            Text(string="+",
              extent=[-120,50; -20,0], style(color=3, rgbcolor={0,0,255})),
            Text(string="-",
              extent=[20,50; 120,0], style(color=3, rgbcolor={0,0,255})),
            Line(points=[50,0; -50,0],   style(color=0, rgbcolor={0,0,0}))));
      equation 
        w = 2*Modelica.Constants.pi*f;
        v = Complex.Constructors.fromPolar(V);
      end VoltageSource;
      
      model VariableVoltageSource 
        extends AC.SinglePhase.Interfaces.Source;
        annotation (Icon(
            Text(string="+",
              extent=[-120,50; -20,0], style(color=3, rgbcolor={0,0,255})),
            Text(string="-",
              extent=[20,50; 120,0], style(color=3, rgbcolor={0,0,255})),
            Line(points=[50,0; -50,0],   style(color=0, rgbcolor={0,0,0}))),
            Diagram);
        Blocks.Interfaces.PolarInput V 
          annotation (extent=[-60,80; -20,120],rotation=-90);
        Modelica.Blocks.Interfaces.RealInput f 
          annotation (extent=[20,80; 60,120], rotation=-90);
      equation 
        w = 2*Modelica.Constants.pi*f;
        v = Complex.Constructors.fromPolar(V);
      end VariableVoltageSource;
      
      model CurrentSource 
        extends AC.SinglePhase.Interfaces.Source;
        parameter Modelica.SIunits.Frequency f = 1;
        parameter AC.Types.PolarCurrent I = AC.Types.PolarCurrent(1, 0);
        annotation (Icon(
            Line(points=[0,-50; 0,50],   style(color=0, rgbcolor={0,0,0})),
            Line(points=[-60,60; 60,60],   style(color=3, rgbcolor={0,0,255})),
            Polygon(points=[60,60; 30,70; 30,50; 60,60],
              style(color=3, rgbcolor={0,0,255}, fillColor=3, rgbfillColor={0,0,255}))));
      equation 
        w = 2*Modelica.Constants.pi*f;
        i = Complex.Constructors.fromPolar(I);
      end CurrentSource;
      
      model VariableCurrentSource 
        extends AC.SinglePhase.Interfaces.Source;
        annotation (Icon(
            Line(points=[0,-50; 0,50],   style(color=0, rgbcolor={0,0,0})),
            Line(points=[-60,60; 60,60],   style(color=3, rgbcolor={0,0,255})),
            Polygon(points=[60,60; 30,70; 30,50; 60,60],
              style(color=3, rgbcolor={0,0,255}, fillColor=3, rgbfillColor={0,0,255}))),
            Diagram);
        Blocks.Interfaces.PolarInput I 
          annotation (extent=[-60,80; -20,120],rotation=-90);
        Modelica.Blocks.Interfaces.RealInput f 
          annotation (extent=[20,80; 60,120], rotation=-90);
      equation 
        w = 2*Modelica.Constants.pi*f;
        i = Complex.Constructors.fromPolar(I);
      end VariableCurrentSource;
    end Sources;
  end SinglePhase;
  
  package Types 
    
    record Reference 
      Modelica.SIunits.Angle phi;
      function equalityConstraint 
        input Reference ref1;
        input Reference ref2;
        output Real residue[0];
      algorithm 
        assert(abs(ref1.phi - ref2.phi) < 1E-6*2*Modelica.Constants.pi, "Reference angles should be equal!");
      end equalityConstraint;
    end Reference;
    
    record Complex "Record defining a Complex number" 
      Real re "Real part of complex number";
      Real im "Imaginary part of complex number";
      
      annotation (Documentation(info="<html>
<p>
This record defines a complex number consisting of a real
and an imaginary part. Additionally, various utility functions
are provided in the record to operate on instances of this record.
Example (note: \"j\" in the comments is defined as j=sqrt(-1)):
</p>
<pre>
   <b>import</b> Modelica.Utilities.Streams;
   <b>import</b> Modelica_Control.Complex;
   <b>import</b> Cmplx = Modelica_Control.Complex.Constructors.default;
   // c1 = 2 + 3j
   Complex c1=Cmplx(re=2, im=3);
   // c2 = 3 + 4j
   Complex c2=Cmplx(3,4);
   // c3 = c1 + c2
   c3 := Complex.'+'(c1, c2);
   Streams.print(\"c3 = \" + Complex.'String'(c3));
   Streams.print(\"c3 = \" + Complex.'String'(c3,\"i\"));
   // This gives the following print-out:
   c3 = 5 + 7j
   c3 = 5 + 7i
</pre>
<p>
The utility functions are written in such a way that it
is convenient to work with them, once operator overloading
is provided in Modelica and Modelica tools. Example:
</p>
<pre>
   // Assume operator overloading is available (this is not yet the case):
   // c4 = -2 + 5*j
   Complex j  = Complex(0,1);
   Complex c4 = -2 + 5*j;
   // A Modelica tool will transform the statement of c4 into
   Complex c4 = Complex.'+'( Complex.Constructors.fromInteger(-2), 
                    Complex.'*'(Complex.Constructors.fromInteger(5),j)));
</pre>
<p>
The utility functions are implemented so that they can be easily
inlined by a tool. In such a case, the above statement will
not lead to any overhead.
</p>
</html>"),
        uses(Modelica(version="1.6")));
      
      encapsulated package Constructors 
        "Library of constructor/cast functions to a Complex number" 
        import Modelica;
        import AC.Types.Complex;
        extends Modelica.Icons.Library;
        
        function default "Default constructor for a complex number" 
          extends Modelica.Icons.Function;
          input Real re "Real part of complex number";
          input Real im=0 "Imaginary part of complex number";
          output Complex result "Complex number";
        algorithm 
          result.re :=re;
          result.im :=im;
        end default;
        
        function fromReal "Construct complex number from Real number" 
          extends Modelica.Icons.Function;
          input Real re "Real part of complex number";
          output Complex result "Complex number";
        algorithm 
          result.re :=re;
          result.im :=0;
        end fromReal;
        
        function fromInteger "Construct complex number from Integer number" 
          extends Modelica.Icons.Function;
          input Integer re "Real part of complex number";
          output Complex result "Complex number";
        algorithm 
          result.re :=re;
          result.im :=0;
        end fromInteger;
        
        function fromPolar "Construct complex number from polar coordinates" 
          extends Modelica.Icons.Function;
          import AC.Types.Polar;
          input Polar polar "polar representation of complex number";
          output Complex result "Complex number";
        algorithm 
          result.re :=polar.len*cos(polar.phi);
          result.im :=polar.len*sin(polar.phi);
        end fromPolar;
      end Constructors;
      
      encapsulated package Examples 
        "Library demonstrating the usage of complex numbers" 
        import Modelica;
        import AC.Types.Complex;
        import Cmplx = AC.Types.Complex.Constructors.default;
        extends Modelica.Icons.Library;
        
        function testComplexNumbers 
          import Modelica.Utilities.Streams;
          extends Modelica.Icons.Function;
        protected 
          Complex c1=Cmplx(re=2, im=3);
          Complex c2=Cmplx(re=3, im=4);
          Complex c3;
          Real z3;
          constant Real eps = 1E-5;
          constant Complex c3_01=Cmplx(re= 0,    im=  0) "0";
          constant Complex c3_02=Cmplx(re= 1,    im=  0) "1";
          constant Complex c3_03=Cmplx(re= 0,    im=  1) "j";
          constant Complex c3_04=Cmplx(re=-2,    im= -3) "-1*";
          constant Complex c3_05=Cmplx(re=-3,    im=  2) "j*";
          constant Complex c3_06=Cmplx(re= 3,    im= -2) "-j*";
          constant Complex c3_07=Cmplx(re= 5,    im=  7) "+";
          constant Complex c3_08=Cmplx(re=-1,    im= -1) "-";
          constant Complex c3_09=Cmplx(re=-6,    im= 17) "*";
          constant Complex c3_10=Cmplx(re= 0.72, im=  0.04) "/";
          constant Real z3_11 = 3.605550 "abs";
          constant Real z3_12 = 0.982794 "arg";
          constant Complex c3_13=Cmplx(re= 2,       im= -3) "conj";
          constant Complex c3_14=Cmplx(re= 1.67415, im=  0.895977) "sqrt";
          constant Complex c3_15=Cmplx(re=-7.31511, im=  1.042740) "exp";
          constant Complex c3_16=Cmplx(re= 1.28247, im=  0.982794) "log";
          constant Complex c3_17=Cmplx(re= 9.15450, im= -4.168910) "sin";
          constant Complex c3_18=Cmplx(re=-4.18963, im= -9.109230) "cos";
        algorithm 
          Streams.print("c1 = " + Complex.'String'(c1));
          Streams.print("c2 = " + Complex.'String'(c2));
          
          c3:=Complex.'0'();
          Streams.print("c3 = 0 + 0*j = " + Complex.'String'(c3));
          assert(c3.re==c3_01.re and c3.im==c3_01.im,"Error.0");
          
          c3:=Complex.'1'();
          Streams.print("c3 = 1 + 0*j = " + Complex.'String'(c3));
          assert(c3.re==c3_02.re and c3.im==c3_02.im,"Error.1");
          
          c3:=Complex.'j'();
          Streams.print("c3 = 0 + 1*j = " + Complex.'String'(c3));
          assert(c3.re==c3_03.re and c3.im==c3_03.im,"Error.j");
          
          c3:=Complex.'-1*'(c1);
          Streams.print("c3 = -c1 = " + Complex.'String'(c3));
          assert(abs(c3.re-c3_04.re)<eps and abs(c3.im-c3_04.im)<eps,"Error.-1*");
          
          c3:=Complex.'j*'(c1);
          Streams.print("c3 = j*c1 = " + Complex.'String'(c3));
          assert(abs(c3.re-c3_05.re)<eps and abs(c3.im-c3_05.im)<eps,"Error.j*");
          
          c3:=Complex.'-j*'(c1);
          Streams.print("c3 = -j*c1 = " + Complex.'String'(c3));
          assert(abs(c3.re-c3_06.re)<eps and abs(c3.im-c3_06.im)<eps,"Error.-j*");
          
          c3:=Complex.'+'(c1, c2);
          Streams.print("c3 = c1 + c2 = " + Complex.'String'(c3));
          assert(abs(c3.re-c3_07.re)<eps and abs(c3.im-c3_07.im)<eps,"Error.+");
          
          c3:=Complex.'-'(c1, c2);
          Streams.print("c3 = c1 - c2 = " + Complex.'String'(c3));
          assert(abs(c3.re-c3_08.re)<eps and abs(c3.im-c3_08.im)<eps,"Error.-");
          
          c3:=Complex.'*'(c1, c2);
          Streams.print("c3 = c1 * c2 = " + Complex.'String'(c3));
          assert(abs(c3.re-c3_09.re)<eps and abs(c3.im-c3_09.im)<eps,"Error.*");
          
          c3:=Complex.'/'(c1, c2);
          Streams.print("c3 = c1 / c2 = " + Complex.'String'(c3));
          assert(abs(c3.re-c3_10.re)<eps and abs(c3.im-c3_10.im)<eps,"Error./");
          
          z3:=Complex.'abs'(c1);
          Streams.print("z3 = abs(c1) = " + String(z3));
          assert(abs(z3-z3_11)<eps,"Error.abs");
          
          z3:=Complex.arg(c1);
          Streams.print("z3 = arg(c1) = " + String(z3));
          assert(abs(z3-z3_12)<eps,"Error.arg");
          
          c3:=Complex.conj(c1);
          Streams.print("c3 = conj(c1) = " + Complex.'String'(c3));
          assert(abs(c3.re-c3_13.re)<eps and abs(c3.im-c3_13.im)<eps,"Error.conj");
          
          c3:=Complex.'sqrt'(c1);
          Streams.print("c3 = sqrt(c1) = " + Complex.'String'(c3));
          assert(abs(c3.re-c3_14.re)<eps and abs(c3.im-c3_14.im)<eps,"Error.sqrt");
          
          c3:=Complex.'exp'(c1);
          Streams.print("c3 = exp(c1) = " + Complex.'String'(c3));
          assert(abs(c3.re-c3_15.re) <eps and abs(c3.im-c3_15.im)<eps,"Error.exp");
          
          c3:=Complex.'log'(c1);
          Streams.print("c3 = log(c1) = " + Complex.'String'(c3));
          assert(abs(c3.re-c3_16.re)<eps and abs(c3.im-c3_16.im)<eps,"Error.log");
          
          c3:=Complex.'sin'(c1);
          Streams.print("c3 = sin(c1) = " + Complex.'String'(c3));
          assert(abs(c3.re-c3_17.re)<eps and abs(c3.im-c3_17.im)<eps,"Error.sin");
          
          c3:=Complex.'cos'(c1);
          Streams.print("c3 = cos(c1) = " + Complex.'String'(c3));
          assert(abs(c3.re-c3_18.re)<eps and abs(c3.im-c3_18.im)<eps,"Error.cos");
        end testComplexNumbers;
      end Examples;
      
      encapsulated function '0' "Null element ('0' + c = c; '0'*c ='0')" 
        import Modelica;
        import AC.Types.Complex;
        extends Modelica.Icons.Function;
        output Complex c;
      algorithm 
        c := Complex.Constructors.default(re=0,im=0);
      end '0';
      
      encapsulated function '1' "Identity element ('1' * c = c)" 
        import Modelica;
        import AC.Types.Complex;
        extends Modelica.Icons.Function;
        output Complex c;
      algorithm 
        c := Complex.Constructors.default(re=1,im=0);
      end '1';
      
      encapsulated function 'j' "Sqrt(-1)" 
        import Modelica;
        import AC.Types.Complex;
        extends Modelica.Icons.Function;
        output Complex c;
      algorithm 
        c := Complex.Constructors.default(re=0,im=1);
      end 'j';
      
      encapsulated function '-1*' "Unary minus (multiply complex number by -1)" 
        import Modelica;
        import AC.Types.Complex;
        extends Modelica.Icons.Function;
        input Complex c1;
        output Complex c2 "= -c1";
      algorithm 
        c2 := Complex(-c1.re, -c1.im);
      end '-1*';
      
      encapsulated function 'j*' "Unary multiply complex number by j" 
        import Modelica;
        import AC.Types.Complex;
        extends Modelica.Icons.Function;
        input Complex c1;
        output Complex c2 "= j*c1";
      algorithm 
        c2 := Complex(-c1.im, c1.re);
      end 'j*';
      
      encapsulated function '-j*' "Unary multiply complex number by -j" 
        import Modelica;
        import AC.Types.Complex;
        extends Modelica.Icons.Function;
        input Complex c1;
        output Complex c2 "= -j*c1";
      algorithm 
        c2 := Complex(c1.im, -c1.re);
      end '-j*';
      
      encapsulated function '+' "Add two complex numbers" 
        import Modelica;
        import AC.Types.Complex;
        extends Modelica.Icons.Function;
        input Complex c1;
        input Complex c2;
        output Complex c3 "= c1 + c2";
      algorithm 
        c3 := Complex(c1.re + c2.re, c1.im + c2.im);
      end '+';
      
      encapsulated function '-' "Subtract two complex numbers" 
        import Modelica;
        import AC.Types.Complex;
        extends Modelica.Icons.Function;
        input Complex c1;
        input Complex c2;
        output Complex c3 "= c1 - c2";
      algorithm 
        c3 := Complex(c1.re - c2.re, c1.im - c2.im);
      end '-';
      
      encapsulated function '*' "Multiply two complex numbers" 
        import Modelica;
        import AC.Types.Complex;
        extends Modelica.Icons.Function;
        input Complex c1;
        input Complex c2;
        output Complex c3 "= c1*c2";
      algorithm 
        c3 := Complex(c1.re*c2.re - c1.im*c2.im, c1.re*c2.im + c1.im*c2.re);
      end '*';
      
      encapsulated function '/' "Divide two complex numbers" 
        import Modelica;
        import AC.Types.Complex;
        extends Modelica.Icons.Function;
        input Complex c1;
        input Complex c2;
        output Complex c3 "= c1/c2";
      algorithm 
        c3 := Complex( ( c1.re*c2.re + c1.im*c2.im)/(c2.re^2 + c2.im^2),
                       (-c1.re*c2.im + c1.im*c2.re)/(c2.re^2 + c2.im^2));
      end '/';
      
      encapsulated function '==' 
        "Test whether two complex numbers are identical" 
        import Modelica;
        import AC.Types.Complex;
        extends Modelica.Icons.Function;
        input Complex c1;
        input Complex c2;
        output Boolean result "c1 == c2";
      algorithm 
        result := c1.re == c2.re and c1.im == c2.im;
      end '==';
      
      encapsulated function 'String' 
        "Transform Complex number into a String representation" 
        import Modelica;
        import AC.Types.Complex;
        extends Modelica.Icons.Function;
        input Complex c 
          "Complex number to be transformed in a String representation";
        input String name="j" 
          "Name of variable representing sqrt(-1) in the string";
        input Integer significantDigits = 6 
          "Number of significant digits that are shown";
        output String s="";
      protected 
        String name2=if name=="" then "j" else name;
      algorithm 
        s := String(c.re,significantDigits=significantDigits);
        if c.im <> 0 then
           if c.im > 0 then
              s := s + " + ";
           else
              s := s + " - ";
           end if;
           s := s + String(abs(c.im),significantDigits=significantDigits) + name2;
        end if;
      end 'String';
      
      encapsulated function toPolar 
        "Transform Complex number into polar representation" 
        import Modelica;
        import AC.Types.Complex;
        import AC.Types.Polar;
        extends Modelica.Icons.Function;
        input Complex c 
          "Complex number to be transformed in a polar representation";
        output Polar polar;
      algorithm 
        polar.len := Complex.'abs'(c);
        polar.phi := Complex.arg(c);
      end toPolar;
      
      encapsulated function 'Real' "Extract real part from Complex number" 
        import Modelica;
        import AC.Types.Complex;
        extends Modelica.Icons.Function;
        input Complex c;
        output Real result;
      algorithm 
        result := c.re;
      end 'Real';
      
      encapsulated function 'Imag' "Extract imaginary part from Complex number" 
        import Modelica;
        import AC.Types.Complex;
        extends Modelica.Icons.Function;
        input Complex c;
        output Real result;
      algorithm 
        result := c.im;
      end 'Imag';
      
      encapsulated function 'abs' "Absolute value of complex number" 
        import Modelica;
        import AC.Types.Complex;
        extends Modelica.Icons.Function;
        input Complex c;
        output Real result "= abs(c)";
      algorithm 
        result := sqrt(c.re^2 + c.im^2);
      end 'abs';
      
      encapsulated function arg "Phase angle of complex number" 
        import Modelica;
        import Modelica.Math;
        import Modelica.Constants;
        import AC.Types.Complex;
        extends Modelica.Icons.Function;
        input Complex c;
        output Modelica.SIunits.Angle result "= phase angle of c";
      algorithm 
        result := if Complex.'abs'(c)<Constants.small then 0 else Math.atan2(c.im, c.re);
      end arg;
      
      encapsulated function conj "Conjugate of complex number" 
        import Modelica;
        import AC.Types.Complex;
        extends Modelica.Icons.Function;
        input Complex c1;
        output Complex c2;
      algorithm 
        c2 := Complex( c1.re, -c1.im);
      end conj;
      
      encapsulated function 'sqrt' "Square root of complex number" 
        import Modelica;
        import Modelica.Math;
        import AC.Types.Complex;
        extends Modelica.Icons.Function;
        input Complex c1;
        output Complex c2;
      algorithm 
        c2 := Complex(sqrt(Complex.'abs'(c1))*Math.cos(Complex.arg(c1)/2),
                      sqrt(Complex.'abs'(c1))*Math.sin(Complex.arg(c1)/2));
      end 'sqrt';
      
      encapsulated function 'exp' "Exponential of complex number" 
        import Modelica;
        import Modelica.Math;
        import AC.Types.Complex;
        extends Modelica.Icons.Function;
        input Complex c1;
        output Complex c2;
      algorithm 
        c2 := Complex(Math.exp(c1.re)*Math.cos(c1.im), Math.exp(c1.re)*Math.sin(c1.im));
      end 'exp';
      
      encapsulated function 'log' "Logarithm of complex number" 
        import Modelica;
        import Modelica.Math;
        import AC.Types.Complex;
        extends Modelica.Icons.Function;
        input Complex c1;
        output Complex c2;
      algorithm 
        c2 := Complex(Math.log(Complex.'abs'(c1)), Complex.arg(c1));
      end 'log';
      
      encapsulated function 'sin' "Sine of complex number" 
        import Modelica;
        import AC.Types.Complex;
        extends Modelica.Icons.Function;
        input Complex c1;
        output Complex c2;
      algorithm 
        c2 := Complex.'/'(
                Complex.'-'(Complex.'exp'(Complex(-c1.im, +c1.re)),
                            Complex.'exp'(Complex(+c1.im, -c1.re))),
                Complex(0,2));
      end 'sin';
      
      encapsulated function 'cos' "Cosine of complex number" 
        import Modelica;
        import AC.Types.Complex;
        extends Modelica.Icons.Function;
        input Complex c1;
        output Complex c2;
      algorithm 
        c2 := Complex.'/'(
                Complex.'+'(Complex.'exp'(Complex(-c1.im, +c1.re)),
                            Complex.'exp'(Complex(+c1.im, -c1.re))),
                Complex(2,0));
      end 'cos';
      
      encapsulated function 'sum' "sum of complex numbers" 
      // useless: Dymola cannot generate code for arrays of records with "unknown" size!
        import Modelica;
        import AC.Types.Complex;
        extends Modelica.Icons.Function;
        input Complex c1[:];
        output Complex c2=c1[1];
      algorithm 
        for i in 2:size(c1, 1) loop
          c2 := Complex.'+'(c2, c1[i]);
        end for;
      end 'sum';
    end Complex;
    
    record ComplexVoltage = Complex (
        redeclare Modelica.SIunits.Voltage re,
        redeclare Modelica.SIunits.Voltage im);
    
    record ComplexCurrent = Complex (
        redeclare Modelica.SIunits.Current re,
        redeclare Modelica.SIunits.Current im);
    
    record ComplexImpedance=Complex (
        redeclare Modelica.SIunits.Resistance re,
        redeclare Modelica.SIunits.Reactance im);
    
    record ComplexAdmittance=Complex (
        redeclare Modelica.SIunits.Conductance re,
        redeclare Modelica.SIunits.Susceptance im);
    
    record ComplexPower=Complex (
        redeclare Modelica.SIunits.ActivePower re,
        redeclare Modelica.SIunits.ReactivePower im);
    
    record ComplexMagneticFlux = Complex (
        redeclare Modelica.SIunits.MagneticFlux re,
        redeclare Modelica.SIunits.MagneticFlux im);
    
    record Polar 
      Real len;
      Modelica.SIunits.Angle phi;
    end Polar;
    
    record PolarVoltage =   Polar (redeclare Modelica.SIunits.Voltage len);
    
    record PolarCurrent =   Polar (redeclare Modelica.SIunits.Current len);
    
    record PolarImpedance =   Polar (redeclare Modelica.SIunits.Impedance len);
    
    record PolarAdmittance =   Polar (redeclare Modelica.SIunits.Admittance len);
    
    record PolarPower =   Polar (redeclare Modelica.SIunits.ApparentPower len);
    
    record PolarMagneticFlux = Polar (redeclare Modelica.SIunits.MagneticFlux 
          len);
  end Types;
end AC;
